import numpy as np
import cv2
import os
from picamera.array import PiRGBArray
from picamera import PiCamera
import time


class DeteccionFacial():
    
    def __init__(self, cascade_fn="./cascades/haarcascade_frontalface_alt2.xml", scaleFactor=1.2, minNeighbors=5, minSize=(30,30)):
        if not os.path.exists(cascade_fn):
            raise IOError("Archivo xml no valido=%s." % cascade_fn)
        self.cascade = cv2.CascadeClassifier(cascade_fn)
        self.scaleFactor = scaleFactor
        self.minNeighbors = minNeighbors
        self.minSize = minSize

    
    def detectar(self, imagen):
        if np.ndim(imagen)==3:
            imagen = cv2.cvtColor(imagen, cv2.COLOR_BGR2GRAY)
        imagen = cv2.equalizeHist(imagen)
        caras = self.cascade.detectMultiScale(imagen, scaleFactor = self.scaleFactor, minNeighbors = self.minNeighbors, minSize = self.minSize)
        if len(caras) == 0:
            return np.ndarray((0,))
        caras[:, 2:] += caras[:, :2]
        return caras
    
    def numeroCaras(self, imagen):
        caras = self.detectar(imagen)
        return len(caras)

    def delimitar(self,img, caras):
        Salida = img.copy()
        for i, r in enumerate(caras):
            x0, y0, x1, y1 = r  
            cv2.rectangle(Salida, (x0,y0),(x1,y1),(0,255,0),1)
        return Salida
    
    def cortar(self, img, caras, j):
        for i, r in enumerate(caras):
            x0, y0, x1, y1 = r  
            face = img[y0:y1,x0:x1]
            nombre = str(j)+".pgm"
            cv2.imwrite(nombre, face)
        
        
class Camara():
    def __init__(self):
        self.camera = PiCamera()
        self.camera.resolution = (320, 240)
        self.camera.framerate = 32
        self.rawCapture = PiRGBArray(self.camera, size=(320, 240))
        time.sleep(0.1)
        
    def __del__(self):
        self.camera.close()
        
    def capturarImagen(self):
        for frame in self.camera.capture_continuous(self.rawCapture, format = "bgr", use_video_port = True):
            imagen = frame.array
            self.rawCapture.truncate(0)
            return imagen
        
    def transmitir(self, image):
        ret, jpeg = cv2.imencode('.jpeg', image)
        return jpeg.tostring()
        