import numpy as np
import cv2
import os



class DeteccionFacial():
    
    def __init__(self, cascade_fn="./cascades/haarcascade_frontalface_alt2.xml", scaleFactor=1.2, minNeighbors=5, minSize=(30,30), cameraId = 0):
        if not os.path.exists(cascade_fn):
            raise IOError("Archivo xml no encontrado=%s." % cascade_fn)
        self.cascade = cv2.CascadeClassifier(cascade_fn)
        self.scaleFactor = scaleFactor
        self.minNeighbors = minNeighbors
        self.minSize = minSize
        self.cameraId = cameraId
    
    def detectar(self, imagen):
        if np.ndim(imagen)==3:
            imagen = cv2.cvtColor(imagen, cv2.COLOR_BGR2GRAY)
        imagen = cv2.equalizeHist(imagen)
        caras = self.cascade.detectMultiScale(imagen, scaleFactor = self.scaleFactor, minNeighbors = self.minNeighbors, minSize = self.minSize)
        if len(caras) == 0:
            return np.ndarray((0,))
        caras[:, 2:] += caras[:, :2]
        return caras

    def numeroCaras(self, imagen):
        caras = self.detectar(imagen)
        return len(caras)
    
    def delimitar(self,img, caras):
        Salida = img.copy()
        for i, r in enumerate(caras):
            x0, y0, x1, y1 = r  
            cv2.rectangle(Salida, (x0,y0),(x1,y1),(0,255,0),1)
        return Salida
    
    def cortar(self, img, caras):
        for j in range(0, 11):
            for i, r in enumerate(caras):
                x0, y0, x1, y1 = r  
                face = img[y0:y1,x0:x1]
                nombre = "/home/tebo/Desktop/faces/" + str(j)+".pgm"
                cv2.imwrite(nombre, face)
        
        
class Camara():
    def __init__(self, camaraId):
        self.camara = cv2.VideoCapture(camaraId)
        
    def __del__(self):
        print "CamaraDestruida"
        self.camara.release()
        
    def capturarImagen(self):
        ret, frame = self.camara.read()
        return frame
    
    def transmitir(self, image):
        ret, jpeg = cv2.imencode('.jpeg', image)
        return jpeg.tostring()


if __name__ == '__main__':

    import time

    c = Camara(0)
    time.sleep(0.1)
    d = DeteccionFacial(cascade_fn="haarcascade_frontalface_alt2.xml")
    cv2.namedWindow('ReconocimientoFacial', cv2.WINDOW_AUTOSIZE)
    cv2.moveWindow('ReconocimientoFacial', 600, 100)
    while True:
        captura = c.capturarImagen()
        #img = cv2.resize(captura, (captura.shape[1] / 2, captura.shape[0] / 2), interpolation=cv2.INTER_CUBIC)
        caras = d.detectar(captura)
        salida = d.delimitar(captura, caras)
        #img1 = cv2.resize(salida, (salida.shape[1] * 2, salida.shape[0] * 2), interpolation=cv2.INTER_CUBIC)
        cv2.imshow('ReconocimientoFacial', salida)
                # Show image & exit on escape:
        ch = cv2.waitKey(10)
        if ch == 27:
            break
    c = None


